﻿Imports ThinkGeo
Imports ThinkGeo.MapSuite
Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.DesktopEdition
Imports System.IO

Public Class Shell
  Private Const ROOT_FOLDER As String = ".\"
  Private Const ISO_FILE As String = "ISO.grd"
  Private Const RANGE_FILE As String = "RANGES.xml"

  Private Sub Shell_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
    Map.MapUnit = GeographyUnit.Meter
    Map.ZoomLevelSnapping = ZoomLevelSnappingMode.None

    Dim ranges As Ranges = GetRanges(RANGE_FILE, 100)

    RenderGridCells(ISO_FILE, RANGE_FILE, ranges)
    LoadIsoLines(ISO_FILE, ranges)

    Map.Refresh()
  End Sub

  Private Function GetRanges(pathAndFile As String, multiplier As Integer) As Ranges
    Dim xd As New Xml.XmlDocument
    Dim ranges As New Ranges

    xd.Load(pathAndFile)

    For Each xnParameters As Xml.XmlNode In xd.ChildNodes

      For Each xnZones As Xml.XmlNode In xnParameters.ChildNodes
        For Each xnZone As Xml.XmlNode In xnZones.ChildNodes
          Dim rng As Range = New Range(xnZone)
          ranges.Add(rng)
        Next
      Next
    Next

    Return ranges
  End Function

  Private Sub RenderGridCells(isoGrdFileName As String, rangesXmlFileName As String, ranges As Ranges)
    Dim pathAndFile As String = Path.Combine(ROOT_FOLDER, isoGrdFileName)

    Dim gfl As New GridFeatureLayer(pathAndFile)
    Dim features As System.Collections.ObjectModel.Collection(Of Feature) = Nothing

    gfl.Open()
    features = gfl.QueryTools.GetAllFeatures(ReturningColumnsType.AllColumns)
    gfl.Close()

    Dim lo As New LayerOverlay
    Map.Overlays.Add(lo)

    Dim imfl As New InMemoryFeatureLayer

    With imfl
      .Open()
      .Columns.Add(New FeatureSourceColumn("RGB"))
      .Close()

      With .ZoomLevelSet.ZoomLevel01
        Dim c As Color
        Dim gc As GeoColor
        Dim valueStyle As New ValueStyle
        Dim gsb As GeoSolidBrush
        Dim gp As GeoPen = New GeoPen(GeoColor.StandardColors.Transparent)

        valueStyle.ColumnName = "RGB"

        c = Drawing.Color.Transparent

        gc = GeoColor.FromArgb(c.A, c.R, c.G, c.B)
        gsb = New GeoSolidBrush(gc)
        valueStyle.ValueItems.Add(New ValueItem(Color.Transparent.ToArgb, New AreaStyle(gp, gsb)))

        For Each rng As Range In ranges
          c = System.Drawing.Color.FromArgb(rng.Rgb)
          gc = GeoColor.FromArgb(c.A, c.R, c.G, c.B)
          gsb = New GeoSolidBrush(gc)
          valueStyle.ValueItems.Add(New ValueItem(rng.Rgb, New AreaStyle(gp, gsb)))
        Next

        .CustomStyles.Add(valueStyle)

        .ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
      End With
    End With

    lo.Layers.Add("CELL", imfl)

    For Each f As Feature In features
      Dim v As Double = CDbl(f.ColumnValues(gfl.DataValueColumnName))
      Dim rgb As Integer = ranges.RgbForValue(v)

      f.ColumnValues.Add("RGB", rgb.ToString)
      imfl.InternalFeatures.Add(f.Id, f)
    Next

    gfl = Nothing

    imfl.Open()
    Dim ext As RectangleShape = imfl.GetBoundingBox
    imfl.Close()

    ext.ScaleUp(10)

    Map.CurrentExtent = ext
  End Sub

  Private Sub LoadIsoLines(isoGrdFileName As String, ranges As Ranges)
    Dim lo As LayerOverlay = CType(Map.Overlays(0), LayerOverlay)
    Dim imfl As New InMemoryFeatureLayer

    Dim isoLineLevels As New List(Of Double)
    Dim isoLineLayer As ThinkGeo.MapSuite.Core.GridIsoLineLayer = Nothing
    Dim features As System.Collections.ObjectModel.Collection(Of ThinkGeo.MapSuite.Core.Feature)

    With imfl
      .ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.DarkBlue, 1, False)
      .ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
    End With

    isoLineLevels.Add(Double.MinValue)

    For Each rng As Range In ranges
      isoLineLevels.Add(rng.Value)
    Next

    isoLineLevels.Add(Double.MaxValue)

    Dim pathAndFile As String = Path.Combine(ROOT_FOLDER, isoGrdFileName)

    isoLineLayer = New ThinkGeo.MapSuite.Core.GridIsoLineLayer(pathAndFile, isoLineLevels)

    isoLineLayer.Open()
    features = isoLineLayer.GetIsoLineFeatures
    isoLineLayer.Close()

    For Each f As Feature In features
      Dim v As Double = CDbl(f.ColumnValues(isoLineLayer.DataValueColumnName))
      Dim rgb As Integer = ranges.RgbForValue(v)
      f.ColumnValues.Add("RGB", rgb)

      imfl.InternalFeatures.Add(f)
    Next

    lo.Layers.Add("ISO", imfl)
  End Sub
End Class
